//******************************************************************************
// Test fixture for CSE 370 - Lab 7 09au Lab 1
//******************************************************************************
`timescale 1ns / 1ns

`define SIMULATION
// `define TWO_BUTTON

module lab7_tf();
   wire [3:0] KEY;      // Pushbutton keys
   wire [6:0] HEX0;	// 7-seg displays
   wire [6:0] HEX1;	// 7-seg displays
   wire [6:0] HEX2;	// 7-seg displays
   wire [6:0] HEX3;	// 7-seg displays
   wire [7:0] LEDG;     // Green LEDs

   reg clk;
   reg ext_reset_n, reset_n, start_n, stop_n; // All active low signals
   assign KEY[0] = reset_n;
   assign KEY[3] = 1'b1;
   
`ifdef TWO_BUTTON
   assign KEY[1] = start_n;
   assign KEY[2] = stop_n;
`else
   assign KEY[1] = start_n & stop_n;
   assign KEY[2] = 1'b1;
`endif		   

localparam DUT_RESET=0, DUT_START=1, DUT_WAITING=2, DUT_STOPPING=3, DUT_IDLE=4;
   reg [2:0]  state, next_state;
   reg [31:0] sleep_timer, next_sleep_length, idle_timer;
   integer    test_count;
   wire [3:0] d0, d1, d2, d3;
   
 
   reaction_timer l7
     (.clk (clk),
	   .reset(~KEY[0]),
	   .start_stop(~KEY[1]),
	   .hex0(HEX0),
	   .hex1(HEX1),
	   .hex2(HEX2),
	   .hex3(HEX3),
	   .leds(LEDG)
	   );
	   
   initial begin
      clk = 1;
      ext_reset_n = 0;  
      // Reset
      #20 ext_reset_n = 1;
   end // initial begin
	   
   always #5 clk = ~clk;

   always @(posedge clk or negedge ext_reset_n) begin
      if(!ext_reset_n) begin
         state <= DUT_RESET;
         sleep_timer <= 0;
         next_sleep_length <= 100;
 //        next_sleep_length <= 5;
         idle_timer <= 25;
         test_count <= 0;
      end else begin
         state <= next_state;
         if(state == DUT_RESET) begin
            idle_timer <= 10;
//            idle_timer <= 5;
         end			
		 if (state == DUT_START) begin
			 idle_timer <= idle_timer - 1;
		 end		 
         if(state == DUT_WAITING) begin	 
			 idle_timer <= 10;
            sleep_timer <= next_sleep_length * (test_count+1);
         end
         if(state == DUT_STOPPING) begin
            sleep_timer <= sleep_timer - 1;
         end
         if(state == DUT_IDLE) begin
            idle_timer <= idle_timer - 1;
         end         
      end
   end

   always @(*) begin
      reset_n = 1;
      start_n = 1;
      stop_n = 1;
	  next_state = state;
      case(state)
        DUT_RESET: begin
           next_state = DUT_START;
           reset_n = 0;
        end
        DUT_START: begin
           start_n = 0; 
		   if (idle_timer == 0) next_state = DUT_WAITING;
        end
        DUT_WAITING: begin
           next_state = LEDG[0] == 1 ? DUT_STOPPING : DUT_WAITING;
        end
        DUT_STOPPING: begin
           if (sleep_timer == 0) begin
              next_state = DUT_IDLE;
              stop_n = 0;
           end else begin
              next_state = DUT_STOPPING;
           end
        end
        DUT_IDLE: begin
           if(idle_timer == 0) begin
              next_state = DUT_RESET;
              test_count = test_count + 1;
              if(test_count == 4) begin
                 $stop;
              end
           end else begin
              next_state = DUT_IDLE;
           end
        end
        default: next_state = DUT_RESET;
      endcase // case (state)
//      $display("[@%5d] %m state=%d next_state=%d rst_n=%b start_n=%b stop_n=%b", $time, state, next_state, rst_n, start_n, stop_n);
   end

   seg7_2dec dec0(.display(HEX0), .num(d0));
   seg7_2dec dec1(.display(HEX1), .num(d1));
   seg7_2dec dec2(.display(HEX2), .num(d2));
   seg7_2dec dec3(.display(HEX3), .num(d3));
   
   always @(posedge clk) begin
	$display("time=%t, KEY0=%b KEY1=%b LEDgreen=%x hex3=%x(%d) hex2=%x(%d) hex1=%x(%d) hex0=%x(%d)",
		$time, KEY[0], KEY[1], LEDG[0], HEX3, d3, HEX2, d2, HEX1, d1, HEX0, d0);
   end

endmodule
